"""Generic utilities module"""

from . import average
from . import extract_ocean_scalar
from . import git
from . import gmeantools
from . import merge
from . import netcdf
from . import xrtools

__all__ = [
    "average",
    "extract_ocean_scalar",
    "git",
    "gmeantools",
    "merge",
    "netcdf",
    "xrtools",
]
