""" Sample db files for demonstration """

import pkg_resources as pkgr

historical = pkgr.resource_filename("gfdlvitals", "resources/historical.db")
picontrol = pkgr.resource_filename("gfdlvitals", "resources/picontrol.db")
